SystemOrganization addCategory: #'Seaside-Development-Core'!
SystemOrganization addCategory: #'Seaside-Development-Filter'!
SystemOrganization addCategory: #'Seaside-Development-Plugins'!

WAErrorHandler subclass: #WADebugErrorHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WADebugErrorHandler commentStamp: 'pmm 5/27/2007 21:47' prior: 0!
I am like WAErrorHandler except that I open a debugger in the image for errors and warnings (not for internal errors).!

!WADebugErrorHandler methodsFor: 'actions' stamp: 'jf 11/12/2008 01:58'!
debugAndResume: anException
	self open: anException.

	"If the user closes the debugger, the process is terminated and the stack unwound.
	There was previously an #ifCurtailed: block around the #open: call above to try to
	catch the process termination so we could display a nice error page but this also
	ended up being called when the user restarted a context in the debugger.
	
	If they Proceed in the debugger, behaviour is platform-dependent. Currently, VW
	should resume the signaling context, so this method will not continue past the call
	to #open:. Squeak currently resumes the context that opened the Debugger, which
	results in the call to #open: returning."

	^ anException isResumable
		ifTrue: [ anException resume ]
		ifFalse: [ self unableToResumeResponse ]! !

!WADebugErrorHandler methodsFor: 'actions' stamp: 'jf 9/19/2008 19:45'!
handleDefault: anException
	^ self debugAndResume: anException! !

!WADebugErrorHandler methodsFor: 'actions' stamp: 'jf 9/30/2009 00:19'!
open: anException 
	GRPlatform current openDebuggerOn: anException! !

!WADebugErrorHandler methodsFor: 'content' stamp: 'pmm 8/23/2011 14:27'!
unableToResumeResponse
	| content |
	content := WAHtmlCanvas builder
		fullDocument: true;
		rootBlock: [ :root | root title: 'Unable to proceed' ];
		render: [ :html |
			html heading level1; with: 'Unable to proceed - hit Refresh to retry'.
			html paragraph: 'Some exceptions cannot be resumed. You will usually see this page if you hit Proceed while debugging one of these exceptions. Your request was aborted but you can retry it by refreshing the page.' ].
	self requestContext respond: [ :response |
		response
			internalError;
			contentType: WAMimeType textHtml;
			nextPutAll: content ]! !

WADebugErrorHandler subclass: #WAWalkbackErrorHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAWalkbackErrorHandler commentStamp: 'jf 10/27/2008 22:44' prior: 0!
I am the default development error handler that displays the stack frames and adds the option to display a debugger.

I delegate the rendering to WAWalkback.

This handler uses Components and depends on having a valid Session. If an error occurs outside of the Session scope it will simply use the superclass behaviour, which should be to open a debugger.!

!WAWalkbackErrorHandler class methodsFor: 'initialization' stamp: 'lr 9/5/2009 10:49'!
initialize
	WAAdmin applicationExceptionHandlingDefaults at: #exceptionHandler put: self! !

!WAWalkbackErrorHandler methodsFor: 'actions' stamp: 'jf 3/3/2009 22:26'!
open: anException 
	| answer |
	self requestContext request isXmlHttpRequest ifTrue: [ ^ super open: anException ].

	"If an error occurs trying to display the walkback component, we just use the super
	behaviour, which should be to open a debugger.."
	answer := true.
	[ self session isNil ifFalse: 
		[ answer := self session presenter call: (WAWalkback current exception: anException).
		"Exception handlers hold the request context because they are used in places where
		the dynamic variable is not present. Since this particular class depends on RenderLoop, 		however, it will be present and we need to update the stored value after doing a call."
		requestContext := WACurrentRequestContext value ] ] 
		on: Error
		do: [ :err | answer := true ].
	answer ifTrue: [ super open: anException ]! !

!GRPackage class methodsFor: '*seaside-development' stamp: 'lr 2/17/2010 15:12'!
seasideDevelopment
	^ self new
		name: 'Seaside-Development';
		description: 'Web-based development tools and debugging support.';
		addDependency: 'Seaside-Tools-Web';
		addDependency: 'Seaside-Flow'; "For WAWalkbackErrorHandler"
		url: #seasideUrl;
		yourself! !

!WARenderContext methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 20:11'!
addHaloVisitor: aHaloVisitor
	visitor := visitor addHaloVisitor: aHaloVisitor! !

!WARenderContext methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 21:04'!
disableHalos
	self properties at: #halosEnabled put: false! !

!WARenderContext methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 21:04'!
enableHalos
	self properties at: #halosEnabled put: true! !

!WARenderContext methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 21:05'!
halosEnabled
	^ self properties at: #halosEnabled ifAbsent: [ false ]! !

WADecoration subclass: #WAToolDecoration
	instanceVariableNames: 'plugins showHalos halos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAToolDecoration class methodsFor: 'testing' stamp: 'lr 9/14/2008 12:54'!
canBeRoot
	^ true! !

!WAToolDecoration methodsFor: 'plugin-halos' stamp: 'lr 9/19/2010 09:42'!
haloFor: aPresenter
	^ halos at: aPresenter ifAbsentPut: [ aPresenter haloClass on: aPresenter ]! !

!WAToolDecoration methodsFor: 'initialization' stamp: 'dkh 11/25/2009 11:34'!
initialize
	super initialize.
	showHalos := WAValueHolder with: false.
	halos := GRPlatform current weakDictionaryOfSize: 20.
	plugins := WAToolPlugin allSubclassesFor: self! !

!WAToolDecoration methodsFor: 'testing' stamp: 'lr 1/3/2008 13:15'!
isGlobal
	^ true! !

!WAToolDecoration methodsFor: 'accessing' stamp: 'dkh 11/25/2009 11:34'!
plugins
	^ plugins! !

!WAToolDecoration methodsFor: 'rendering' stamp: 'jf 2/12/2009 21:05'!
renderChildOn: html
	self showHalos ifTrue: [
		html context addHaloVisitor: (WAHaloVisitor decoration: self context: html context).
		html context enableHalos ].
	self renderNextOn: html.
	html context disableHalos! !

!WAToolDecoration methodsFor: 'rendering' stamp: 'lr 10/28/2007 14:42'!
renderContentOn: html
	self renderChildOn: html.
	self renderToolbarOn: html! !

!WAToolDecoration methodsFor: 'rendering' stamp: 'lr 3/7/2009 14:36'!
renderToolbarOn: html
	html div class: 'wa-toolbar'; with: [
		(self plugins select: [ :each | each isActive ])
			do: [ :each | html render: each ]
			separatedBy: [ html text: ' ' ] ]! !

!WAToolDecoration methodsFor: 'plugin-halos' stamp: 'lr 6/5/2007 21:46'!
showHalos
	^ showHalos contents! !

!WAToolDecoration methodsFor: 'accessing' stamp: 'lr 4/15/2007 18:46'!
states
	^ Array with: showHalos! !

!WAToolDecoration methodsFor: 'plugin-halos' stamp: 'lr 3/24/2008 11:20'!
toggleHalos
	showHalos contents: showHalos contents not! !

!WAToolDecoration methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	WADevelopmentFiles default updateRoot: anHtmlRoot! !

!WAToolDecoration methodsFor: 'updating' stamp: 'jf 2/21/2009 03:20'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	"Create halos for all visible presenters at the end of the action phase so that we
	can properly snapshot their initial values (otherwise they get created in the render
	phase and we have no snapshot of the state for that render)."
	self showHalos ifTrue: [ self visiblePresentersDo: [ :each | self haloFor: each ] ].
	aSnapshot register: halos.
	halos do: [ :each  | each updateStates: aSnapshot ]! !

WAFileLibrary subclass: #WADevelopmentFiles
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WADevelopmentFiles commentStamp: 'jf 10/6/2008 21:27' prior: 0!
CSS and images used by the web-based development tools.

The icons are from the Tango Desktop Project (tango.freedesktop.org), they are licensed under the Creative Commons Attribution Share-Alike license.
!

!WADevelopmentFiles methodsFor: 'uploaded' stamp: 'pmm 2/12/2011 21:28'!
codebrowserPng
	^ #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 213 9 22 18 55 41 59 82 2 72 0 0 1 203 73 68 65 84 56 203 165 147 77 104 19 65 20 199 127 51 221 163 72 22 34 45 226 177 224 41 160 88 43 1 15 130 21 132 120 236 69 208 182 39 175 165 23 21 60 138 32 197 32 245 34 126 128 154 22 138 49 189 91 26 17 77 92 16 237 193 98 211 141 27 76 253 192 213 108 235 16 21 90 145 205 120 48 67 179 109 106 43 190 203 255 13 204 251 207 111 102 222 131 255 12 97 146 226 93 250 128 25 224 216 86 122 120 136 188 169 147 0 151 71 47 157 1 102 14 30 119 216 142 182 18 88 77 189 1 240 237 249 57 182 163 87 79 74 221 172 59 45 141 211 222 84 64 89 21 88 221 147 167 172 10 252 232 122 216 86 95 76 75 6 199 138 28 233 31 6 24 55 4 196 227 113 196 137 165 63 139 125 77 101 77 125 223 39 127 79 50 116 237 25 31 158 140 51 183 16 132 64 135 213 122 31 165 20 245 122 125 195 75 7 65 128 155 73 173 21 123 203 124 45 77 117 0 61 17 3 219 182 177 109 59 82 236 251 62 110 38 197 224 88 145 197 199 25 94 149 151 81 165 28 64 207 200 100 99 246 175 4 230 228 129 116 129 197 124 134 185 183 10 85 202 241 115 255 69 206 159 189 48 11 136 77 9 180 214 20 210 189 12 92 121 138 55 125 135 202 231 239 168 249 7 188 239 26 166 179 205 55 70 8 180 214 100 179 89 126 201 67 204 59 14 239 62 213 89 122 61 5 189 105 118 173 172 0 218 52 161 110 75 16 134 33 187 63 166 121 244 165 155 251 185 28 221 242 37 59 251 110 145 76 38 113 93 151 55 149 5 140 139 92 79 80 173 86 241 60 143 145 201 6 71 59 43 172 238 72 16 30 24 37 145 72 96 89 22 150 101 69 186 120 3 65 44 22 67 107 77 173 86 67 107 141 148 18 173 53 66 8 132 16 173 219 27 17 3 165 20 142 227 108 58 117 198 100 125 24 131 137 155 183 175 159 250 135 41 158 48 201 111 90 157 232 152 121 9 252 18 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray! !

!WADevelopmentFiles methodsFor: 'styles' stamp: 'pmm 7/18/2010 13:05'!
developmentCss
	^ '/* toolbar */
body { margin-bottom: 25px !!important; /*matches toolbar min-height + padding and margin*/}
.wa-toolbar {
	left: 0;
	right: 0;
	bottom: 0;
	z-index: 20;
	padding: 2px;
	position: fixed;
	text-align: left;
	background-color: #eee;
	border-top: 1px solid #aaa;
	min-height: 20px;
}
.wa-toolbar { vertical-align: middle; }
.wa-toolbar, .wa-toolbar * {
	color: #444;
	font-size: 14px;
	font-family: "Helvetica Neue", Helvetica, Verdana, Arial, sans-serif;
}
.wa-toolbar a {
	padding-left: 3px;
	padding-right: 3px;
	text-decoration: none;
}
.wa-toolbar a:hover {
	text-decoration: underline;
}
.wa-toolbar .wa-deprecationtoolplugin {
	color: #ff7000;
}

/* halos */
.wa-halo {
	margin: 4px;
	text-align: left;
	border: 1px solid #aaa;
}

.wa-halo .wa-halo-header {
	padding: 2px 4px ;
	overflow: hidden;
	background-color: #eee;
	border-bottom: 1px solid #aaa;
}
.wa-halo .wa-halo-header,
.wa-halo .wa-halo-header * {
	color: #444;
	font-size: 14px;
	font-family: "Helvetica Neue", Helvetica, Verdana, Arial, sans-serif;
}
.wa-halo .wa-halo-header a {
	text-decoration: none;
}
.wa-halo .wa-halo-header .wa-classnamehaloplugin,
.wa-halo .wa-halo-header .wa-browserhaloplugin,
.wa-halo .wa-halo-header .wa-inspectorhaloplugin,
.wa-halo .wa-halo-header .wa-styleshaloplugin {
	float: left;
	display: block;
	padding-right: 4px;
}
.wa-halo .wa-halo-header .wa-classnamehaloplugin {
	font-weight: bold;
	padding-right: 8px;
}
.wa-halo .wa-halo-header .wa-browserhaloplugin, .window .window-titlebar .wa-browserhaloplugin {
	background: url(codebrowser.png) no-repeat;
}
.wa-halo .wa-halo-header .wa-inspectorhaloplugin, .window .window-titlebar .wa-inspectorhaloplugin {
	background: url(inspector.png) no-repeat;
}
.wa-halo .wa-halo-header .wa-styleshaloplugin, .window .window-titlebar .wa-styleshaloplugin {
	background: url(styleeditor.png) no-repeat;
}
.wa-halo .wa-halo-header .wa-browserhaloplugin a,
.wa-halo .wa-halo-header .wa-inspectorhaloplugin a,
.wa-halo .wa-halo-header .wa-styleshaloplugin a {
	width: 16px;
	outline: none;
	display: block;
	text-indent: -32000px;
	background-position: 0 50%;
}
.wa-halo .wa-halo-header .wa-modehaloplugin {
	float: right;
	display: block;
	padding-right: 4px;
}
.wa-halo .wa-halo-header .wa-modehaloplugin a {
	text-decoration: none;
}
.wa-halo .wa-halo-header .wa-modehaloplugin a:hover {
	text-decoration: underline;
}
.wa-halo .wa-halo-header .wa-modehaloplugin a.active {
	font-weight: bold;
}

.wa-halo .wa-halo-body {
	clear: both;
}
.wa-halo .wa-halo-body .wa-halo-source {
	padding: 2px 4px;
	background-color: #fff;
}
.wa-halo .wa-halo-body .wa-halo-source,
.wa-halo .wa-halo-body .wa-halo-source * {
	color: #444;
	line-height: 1;
	font-size: 12px;
	font-family: Monaco, "Courier New", Courier, serif;
}
.wa-halo .wa-halo-body .wa-halo-source .wa-tag {
	color: navy;
	font-weight: bold;
}
.wa-halo .wa-halo-body .wa-halo-source .wa-attribute-key {
	color: navy;
}
.wa-halo .wa-halo-body .wa-halo-source .wa-attribute-value,
.wa-halo .wa-halo-body .wa-halo-source .wa-attribute-value a {
	color: purple;
	text-decoration: none;
}
.wa-halo .wa-halo-body .wa-halo-source .wa-attribute-value a:hover {
	text-decoration: underline;
}'! !

!WADevelopmentFiles methodsFor: 'uploaded' stamp: 'pmm 2/12/2011 21:28'!
inspectorPng
	^ #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 6 98 75 71 68 0 240 0 240 0 239 52 6 103 27 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 213 11 5 16 15 5 37 253 173 47 0 0 1 246 73 68 65 84 56 203 157 146 77 107 19 81 24 133 159 59 201 36 77 154 102 134 74 82 10 10 181 93 104 26 69 20 193 133 184 20 10 85 119 226 38 191 192 111 169 244 31 136 182 36 65 68 20 116 89 119 85 16 197 77 168 59 65 236 162 84 40 164 137 86 171 150 80 53 141 205 71 39 77 50 51 215 69 72 73 66 82 170 103 119 15 239 61 239 225 188 71 60 121 250 120 49 247 39 119 140 61 64 85 213 231 19 183 38 47 182 144 211 209 187 150 220 35 238 77 223 145 237 162 78 41 235 92 62 191 185 235 118 77 211 59 242 78 33 196 206 35 153 76 118 28 10 133 66 93 133 119 28 180 15 102 50 25 102 95 204 2 144 152 75 0 208 180 107 119 7 223 190 175 146 74 167 57 122 106 140 109 197 139 105 43 184 100 133 212 252 107 38 110 223 140 196 162 247 159 53 254 40 237 14 52 93 35 149 78 115 232 228 24 159 11 94 6 7 2 140 28 8 82 18 62 180 209 113 252 154 62 115 237 198 149 227 93 29 36 230 18 12 135 79 179 82 112 115 233 204 16 190 30 21 211 182 81 29 10 239 150 37 193 35 227 54 75 111 22 0 1 160 116 10 75 122 3 12 13 232 88 182 100 219 52 169 212 44 202 85 139 126 159 155 95 101 181 37 9 69 74 137 148 118 75 48 110 183 11 132 160 92 181 168 214 36 155 91 53 220 170 131 253 253 30 164 16 45 93 112 2 182 105 154 45 78 108 99 131 53 163 138 223 163 162 247 185 176 45 137 223 227 228 211 70 129 125 61 150 157 107 114 174 212 43 234 66 211 116 52 77 231 194 185 243 164 23 222 18 236 19 124 249 153 39 155 43 145 55 42 124 92 89 167 84 174 144 91 122 229 44 21 139 63 26 2 34 26 159 154 177 44 43 210 124 13 195 48 112 56 84 78 156 141 176 188 86 100 203 84 24 236 149 172 47 190 100 248 224 8 0 31 230 223 167 30 62 120 116 88 116 107 216 213 235 151 167 2 129 224 100 51 231 235 245 17 30 13 243 59 155 173 247 101 245 43 130 127 68 44 30 149 13 145 255 18 0 148 88 60 106 1 148 13 131 191 6 140 246 211 127 51 231 46 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray! !

!WADevelopmentFiles methodsFor: 'accessing' stamp: 'lr 3/7/2009 15:14'!
selectorsToInclude
	^ #( developmentCss )! !

!WADevelopmentFiles methodsFor: 'uploaded' stamp: 'pmm 2/12/2011 21:28'!
styleeditorPng
	^ #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 214 4 9 20 4 42 36 195 187 166 0 0 2 112 73 68 65 84 56 203 173 147 221 75 83 113 24 199 63 155 155 110 57 245 164 169 232 24 180 240 173 50 207 48 74 144 68 27 82 209 141 43 147 145 136 226 159 97 36 236 70 130 174 162 232 38 34 42 86 104 26 42 5 74 47 243 5 65 212 110 52 148 48 95 74 156 107 206 236 156 249 50 117 110 167 139 58 232 130 186 144 30 248 193 247 226 249 124 248 61 207 143 31 28 172 146 107 156 213 74 119 119 183 114 16 216 180 15 222 19 184 65 116 67 167 27 228 223 167 211 13 226 31 112 226 126 184 198 89 173 104 84 24 232 183 55 54 10 73 201 41 16 217 37 184 178 66 95 107 171 4 84 44 222 110 89 150 229 96 219 204 204 76 89 93 109 61 85 85 85 212 56 171 105 111 123 169 87 5 157 246 250 122 135 226 245 18 24 25 1 69 225 136 77 132 212 52 94 127 15 140 249 46 86 230 100 103 153 15 23 228 159 160 188 188 92 133 117 64 36 14 224 42 60 40 42 41 73 88 236 233 97 119 99 131 104 56 204 178 94 199 156 205 198 194 217 211 230 230 155 46 163 86 27 135 40 138 108 132 130 60 122 248 88 7 68 0 180 234 112 74 40 4 202 222 78 190 230 230 225 63 83 76 83 83 51 62 159 143 188 188 60 218 59 94 176 24 76 222 108 56 89 89 168 246 169 2 79 48 16 32 45 63 31 157 193 192 252 229 75 248 47 216 185 238 172 197 231 243 97 52 26 233 31 232 227 211 236 23 4 177 250 208 146 144 237 138 17 76 31 79 126 254 166 191 87 193 100 226 104 105 41 19 69 133 92 113 92 99 109 109 13 69 81 88 95 15 50 60 58 202 80 192 138 201 168 195 159 146 105 143 17 124 168 73 170 141 27 44 211 188 21 38 232 26 241 144 62 61 133 167 239 61 130 32 32 203 50 119 238 221 229 227 143 12 66 187 113 108 135 35 49 239 170 3 208 155 37 123 162 152 195 192 173 49 182 36 3 130 55 132 102 106 146 150 251 207 8 71 34 172 104 173 44 109 9 164 101 89 9 200 219 100 202 126 79 140 0 32 204 38 74 244 87 150 204 195 196 235 102 241 246 26 176 228 22 99 200 182 112 42 43 129 213 245 29 62 15 116 72 54 105 201 21 35 8 123 5 207 106 116 206 97 201 201 96 97 122 25 128 236 226 45 178 114 94 49 120 99 202 247 77 41 72 4 200 148 253 30 155 180 228 122 50 249 110 92 21 104 0 28 117 86 49 122 46 208 127 190 193 34 24 73 37 202 46 235 138 159 161 167 43 146 118 40 189 162 203 61 63 254 183 143 161 81 131 163 206 42 238 28 219 113 233 205 146 93 189 85 252 92 188 235 95 240 127 169 159 24 23 11 103 188 240 115 88 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray! !

WAPainterVisitor subclass: #WAHaloVisitor
	instanceVariableNames: 'toolDecoration renderContext visitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAHaloVisitor class methodsFor: 'instance creation' stamp: 'jf 2/12/2009 20:26'!
decoration: aToolDecoration context: aRenderContext
	^ self basicNew
		initializeWithToolDecoration: aToolDecoration renderContext: aRenderContext;
		yourself! !

!WAHaloVisitor methodsFor: 'initialization' stamp: 'jf 2/12/2009 20:25'!
initializeWithToolDecoration: aToolDecoration renderContext: aRenderContext
	self initialize.
	toolDecoration := aToolDecoration.
	renderContext := aRenderContext! !

!WAHaloVisitor methodsFor: 'copying' stamp: 'jf 2/12/2009 21:24'!
postCopyForRenderContext: aRenderContext
	super postCopyForRenderContext: aRenderContext.
	renderContext := aRenderContext.
	visitor := visitor copyForRenderContext: aRenderContext! !

!WAHaloVisitor methodsFor: 'accessing' stamp: 'jf 2/12/2009 20:26'!
renderContext
	^ renderContext! !

!WAHaloVisitor methodsFor: 'accessing' stamp: 'jf 2/12/2009 20:27'!
toolDecoration
	^ toolDecoration! !

!WAHaloVisitor methodsFor: 'visiting' stamp: 'jf 2/13/2009 15:32'!
visitPainter: aPainter 
	| halo |
	(self renderContext halosEnabled and: [ aPainter canHaveHalo ]) 
		ifTrue: [
			halo := self toolDecoration haloFor: aPainter.
			halo renderWithContext: self renderContext visitor: visitor ]
		ifFalse: [ visitor visit: aPainter ]
	! !

!WAHaloVisitor methodsFor: 'accessing' stamp: 'jf 2/12/2009 19:41'!
visitor
	^ visitor! !

!WAHaloVisitor methodsFor: 'accessing' stamp: 'jf 2/12/2009 19:41'!
visitor: anObject
	visitor := anObject! !

!WAPainterVisitor methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 20:12'!
addHaloVisitor: aHaloVisitor
	^ aHaloVisitor
		visitor: self;
		yourself! !

!WAPainter methodsFor: '*seaside-development' stamp: 'jf 2/13/2009 09:24'!
canHaveHalo
	"Return true if this Painter should have a Halo displayed for it when Halos are
	enabled in the development environment."
	
	^ false! !

WAPlugin subclass: #WAHaloPlugin
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAHaloPlugin commentStamp: 'pmm 5/27/2007 07:47' prior: 0!
Abstract root class for all halo tools. They add functionality to the halo toolbar.!

WAHaloPlugin subclass: #WAClassnameHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAClassnameHaloPlugin commentStamp: 'pmm 5/27/2007 17:34' prior: 0!
I display the class name of the component class.!

!WAClassnameHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 11:02'!
priority
	^ 100! !

!WAClassnameHaloPlugin methodsFor: 'rendering' stamp: 'dkh 11/25/2009 11:24'!
renderContentOn: html
	html div class: self cssClass; with: self root className! !

!WAHaloPlugin class methodsFor: 'instance creation' stamp: 'jf 2/15/2010 23:38'!
allSubclassesOn: aTargetObject for: aRequestorObject
	| plugins |
	plugins := self allSubclasses
		collect: [ :each | each on: aTargetObject for: aRequestorObject ].
	^ plugins sorted! !

!WAHaloPlugin class methodsFor: 'instance creation' stamp: 'dkh 11/25/2009 11:45'!
on: aTargetObject for: aRequestorObject
	^ self basicNew initializeRoot: aTargetObject requestor: aRequestorObject! !

!WAHaloPlugin methodsFor: 'initialization' stamp: 'dkh 11/25/2009 11:35'!
initializeRoot: aRootObject requestor: aRequestorObject
	self initializeRequestor: aRequestorObject.
	root := aRootObject.
! !

!WAHaloPlugin methodsFor: 'actions' stamp: 'lr 3/14/2009 09:09'!
open: aComponent onAnswer: aBlock
	| component decoration |
	component := self session presenter.
	decoration := component decorations
		detect: [ :each | each isKindOf: WAToolDecoration ]
		ifNone: [ nil ].
	decoration isNil
		ifTrue: [ ^ super open: aComponent onAnswer: aBlock ].
	component removeDecoration: decoration.
	super open: aComponent onAnswer: [ :value |
		component addDecoration: decoration.
		aBlock value: value ]! !

!WAHaloPlugin methodsFor: 'rendering' stamp: 'jf 11/4/2008 12:22'!
renderContentOn: html
	html div
		class: self cssClass;
		with: [ super renderContentOn: html ]! !

!WAHaloPlugin methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:04'!
root
	^ root! !

WAHaloPlugin subclass: #WAInspectorHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAInspectorHaloPlugin commentStamp: 'pmm 5/27/2007 17:35' prior: 0!
I open an inspector on the component class.!

!WAInspectorHaloPlugin methodsFor: 'actions' stamp: 'dkh 11/25/2009 11:25'!
execute
	self open: (WAInspector current on: self root)! !

!WAInspectorHaloPlugin methodsFor: 'testing' stamp: 'jf 3/3/2009 22:34'!
isActive
	^ WAInspector current notNil! !

!WAInspectorHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:55'!
label
	^ 'Inspector'! !

!WAInspectorHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 11:02'!
priority
	^ 600! !

!WAInspectorHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 01:03'!
tooltip
	^ 'Object Inspector'! !

WAHaloPlugin subclass: #WAModeHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAModeHaloPlugin commentStamp: 'pmm 5/27/2007 17:35' prior: 0!
I control the rendering modes of the halo.!

!WAModeHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 11:02'!
priority
	^ 900! !

!WAModeHaloPlugin methodsFor: 'rendering' stamp: 'lr 3/7/2009 15:20'!
renderContentOn: html
	html div class: self cssClass; with: [
		self requestor modes
			do: [ :each |
				html anchor
					class: 'active' if: self requestor mode = each;
					callback: [ self requestor mode: each ];
					with: (self labelForSelector: each) ]
			separatedBy: [ html text: ' / ' ] ]! !

WAHaloPlugin subclass: #WAStylesHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAStylesHaloPlugin commentStamp: 'pmm 5/27/2007 17:35' prior: 0!
I open the style editor on the component class.!

!WAStylesHaloPlugin methodsFor: 'actions' stamp: 'dkh 11/25/2009 11:26'!
execute
	self open: (WAPluggableTool on: (WAViewer on: self root))! !

!WAStylesHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:56'!
label
	^ 'Styles'! !

!WAStylesHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 11:02'!
priority
	^ 700! !

!WAStylesHaloPlugin methodsFor: 'accessing' stamp: 'pmm 5/16/2007 16:24'!
tooltip
	^ 'CSS Style Editor'! !

WAPlugin subclass: #WAToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAToolPlugin commentStamp: 'pmm 5/27/2007 07:47' prior: 0!
Abstract root class for all development tool bar tools.!

WAToolPlugin subclass: #WAConfigureToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAConfigureToolPlugin commentStamp: 'pmm 5/27/2007 18:33' prior: 0!
I display the configuration dialog for the currently active application!

!WAConfigureToolPlugin methodsFor: 'actions' stamp: 'pmm 11/14/2008 22:26'!
execute
	self open: (WAPluggableTool on: self application configurationComponent)! !

!WAConfigureToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:16'!
label
	^ 'Configure'! !

!WAConfigureToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:56'!
priority
	^ 200! !

WAToolPlugin subclass: #WADeprecationToolPlugin
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WADeprecationToolPlugin commentStamp: 'pmm 5/27/2007 18:33' prior: 0!
I display deprecation warnings.!

!WADeprecationToolPlugin methodsFor: 'actions' stamp: 'pmm 10/11/2008 18:27'!
execute
	self open: (WADeprecation on: filter)! !

!WADeprecationToolPlugin methodsFor: 'initialization' stamp: 'dkh 11/25/2009 11:38'!
initializeRequestor: aRequestorObject
	super initializeRequestor: aRequestorObject.
	filter := self session filters
		detect: [ :each | each isKindOf: WADeprecatedToolFilter ]
		ifNone: [ self session addFilter: WADeprecatedToolFilter new ]! !

!WADeprecationToolPlugin methodsFor: 'testing' stamp: 'lr 9/25/2008 15:14'!
isActive
	^ filter notifications notEmpty! !

!WADeprecationToolPlugin methodsFor: 'accessing' stamp: 'lr 12/30/2008 15:41'!
label
	^ 'Deprecated (' , filter notifications size printString , ')'! !

!WADeprecationToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:59'!
priority
	^ 700! !

WAToolPlugin subclass: #WANewSessionToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WANewSessionToolPlugin commentStamp: 'pmm 5/27/2007 18:32' prior: 0!
I create a new session.!

!WANewSessionToolPlugin class methodsFor: 'accessing' stamp: 'lr 7/25/2011 19:49'!
title
	^ 'New Session'! !

!WANewSessionToolPlugin methodsFor: 'actions' stamp: 'jf 11/3/2009 01:15'!
execute
	self session unregister.
	self requestContext redirectTo: self application url! !

!WANewSessionToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:26'!
label
	^ 'New Session'! !

!WANewSessionToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:56'!
priority
	^ 100! !

WAToolPlugin subclass: #WATerminateToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WATerminateToolPlugin commentStamp: 'pmm 5/27/2007 18:32' prior: 0!
I try to terminate the process handling this session. This can work of you're at 100% CPU.!

!WATerminateToolPlugin methodsFor: 'testing' stamp: 'jf 1/30/2009 17:22'!
isActive
	^ self application preferenceAt: #sessionAllowTermination! !

!WATerminateToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:29'!
label
	^ 'Terminate'! !

!WATerminateToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:57'!
priority
	^ 600! !

!WATerminateToolPlugin methodsFor: 'rendering' stamp: 'jf 2/15/2010 02:11'!
renderContentOn: html
	html anchor
		class: self cssClass;
		url: (html actionUrl
			withField: 'terminate');
		with: self label! !

WAToolPlugin subclass: #WATimingToolPlugin
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WATimingToolPlugin commentStamp: 'pmm 5/27/2007 18:31' prior: 0!
I display render and callback times.!

!WATimingToolPlugin methodsFor: 'initialization' stamp: 'dkh 11/25/2009 11:39'!
initializeRequestor: aRequestorObject
	super initializeRequestor: aRequestorObject.
	filter := self session filters
		detect: [ :each | each isKindOf: WATimingToolFilter ]
		ifNone: [ self session addFilter: WATimingToolFilter new ]! !

!WATimingToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:59'!
priority
	^ 900! !

!WATimingToolPlugin methodsFor: 'rendering' stamp: 'lr 3/7/2009 15:11'!
renderContentOn: html
	html span class: self cssClass; with: [
		html span
			title: 'Callback Time';
			with: filter previousTiming.
		html text: '/'.
		html span
			title: 'Render Time';
			with: filter currentTiming.
		html text: ' ms' ]! !

WAToolPlugin subclass: #WAToggleHalosToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAToggleHalosToolPlugin commentStamp: 'pmm 5/27/2007 18:31' prior: 0!
I toggle halos on and off.!

!WAToggleHalosToolPlugin methodsFor: 'actions' stamp: 'lr 5/11/2007 13:54'!
execute
	 self requestor toggleHalos! !

!WAToggleHalosToolPlugin methodsFor: 'accessing' stamp: 'lr 3/7/2009 15:43'!
label
	^ 'Halos'! !

!WAToggleHalosToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:56'!
priority
	^ 300! !

!WAToolPlugin class methodsFor: 'instance creation' stamp: 'jf 2/15/2010 23:39'!
allSubclassesFor: aRequestorObject
	| plugins |
	plugins := self allSubclasses
		collect: [ :each | each for: aRequestorObject ].
	^ plugins sorted! !

!WAToolPlugin class methodsFor: 'instance creation' stamp: 'dkh 11/25/2009 11:31'!
for: aRequestorObject
	^ self basicNew initializeRequestor: aRequestorObject! !

!WAToolPlugin methodsFor: 'actions' stamp: 'jf 9/30/2009 18:31'!
open: aComponent onAnswer: aBlock
	| component |
	component := self requestor decoratedComponent.
	component removeDecoration: self requestor.
	super open: aComponent onAnswer: [ :value |
		component addDecoration: self requestor.
		aBlock value: value ]! !

WAToolPlugin subclass: #WAValidateToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Plugins'!

!WAValidateToolPlugin commentStamp: 'pmm 5/27/2007 18:31' prior: 0!
I validate and XHTML page by sending it to the W3C validator. I only work if the website is publically available.!

!WAValidateToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:30'!
label
	^ 'XHTML'! !

!WAValidateToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:59'!
priority
	^ 800! !

!WAValidateToolPlugin methodsFor: 'rendering' stamp: 'lr 3/7/2009 15:10'!
renderContentOn: html
	html anchor
		class: self cssClass;
		title: 'Validate XHTML';
		url: 'http://validator.w3.org/check/referer';
		with: self label! !

WASystemConfiguration subclass: #WADevelopmentConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WADevelopmentConfiguration class methodsFor: 'initialization' stamp: 'lr 9/5/2009 10:47'!
initialize
	WAAdmin applicationDefaults addParent: self instance! !

!WADevelopmentConfiguration methodsFor: 'description' stamp: 'jf 3/4/2009 21:38'!
describeOn: config
	config at: #rootDecorationClasses addAll: (Array with: WAToolDecoration)! !

!WADevelopmentConfiguration methodsFor: 'ancestry' stamp: 'jf 1/30/2009 17:40'!
parents
	^ Array with: WARenderLoopConfiguration instance! !

WALabelledFormDialog subclass: #WAViewer
	instanceVariableNames: 'presenter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAViewer class methodsFor: 'instance creation' stamp: 'avi 3/13/2004 01:13'!
on: aPresenter
	^ self new presenter: aPresenter! !

!WAViewer methodsFor: 'accessing' stamp: 'avi 3/13/2004 01:14'!
buttons
	^ #(save)! !

!WAViewer methodsFor: 'actions' stamp: 'jf 9/30/2009 00:19'!
compile: aString 
	GRPlatform current 
		compile: aString
		into: presenter class
		classified: #rendering! !

!WAViewer methodsFor: 'accessing' stamp: 'avi 3/13/2004 01:14'!
model
	^ presenter! !

!WAViewer methodsFor: 'accessing' stamp: 'avi 3/13/2004 01:13'!
presenter: aPresenter
	presenter := aPresenter! !

!WAViewer methodsFor: 'rendering' stamp: 'jf 9/30/2009 01:10'!
renderContentOn: html 
	html heading
		level3;
		with: presenter greaseString.
	super renderContentOn: html! !

!WAViewer methodsFor: 'rendering' stamp: 'pmm 8/13/2010 11:58'!
renderCssOn: html
	html textArea
		rows: 20;
		columns: 80;
		value: presenter style;
		callback: [ :css |
			self compile: 'style ', GRPlatform current newline,
				'	^ ', css printString ]! !

!WAViewer methodsFor: 'accessing' stamp: 'avi 3/13/2004 01:14'!
rows
	^ #(css)! !

!WAViewer methodsFor: 'actions' stamp: 'lr 10/28/2007 14:42'!
save! !

!WAPresenter methodsFor: '*seaside-development' stamp: 'lr 9/19/2010 09:43'!
canHaveHalo
	"Answer true if the presenter should display a halo."

	^ self haloClass notNil! !

!WAPresenter methodsFor: '*seaside-development' stamp: 'lr 9/19/2010 09:43'!
haloClass
	"Answer the halo class to be used to wrap this component."

	^ WAHalo! !

WAComponent subclass: #WATool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WATool commentStamp: 'pmm 10/11/2008 18:58' prior: 0!
A WATool is the common base class for tools called from the development tool bar. Is doesn't do anything but including WADevelopmentFiles.!

WATool subclass: #WADeprecation
	instanceVariableNames: 'table filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WADeprecation class methodsFor: 'instance creation' stamp: 'lr 7/25/2011 19:48'!
on: filter
	^ self new
		filter: filter;
		yourself! !

!WADeprecation methodsFor: 'private' stamp: 'pmm 2/13/2011 10:52'!
buildTable
	^ WATableReport new
		rowPeriod: 1;
		rowColors: #( '#eee' 'white' );
		rows: self notifications;
		columns: (Array
			with: (WAReportColumn selector: #messageText title: 'Deprecated')
			with: (WAReportColumn selector: #details title: 'Details'));
		yourself! !

!WADeprecation methodsFor: 'accessing' stamp: 'lr 12/30/2008 15:37'!
children
	^ Array with: self table! !

!WADeprecation methodsFor: 'accessing' stamp: 'obi 10/8/2008 20:20'!
filter: aFilter
	filter := aFilter! !

!WADeprecation methodsFor: 'accessing' stamp: 'lr 12/30/2008 15:37'!
notifications
	^ filter notifications! !

!WADeprecation methodsFor: 'rendering' stamp: 'pmm 10/11/2008 18:52'!
renderContentOn: html
	html div
		class: 'tool';
		with: [
			html render: self table.
			html anchor
				callback: [ self resetDeprecationWarnings ];
				with: 'Reset deprecation warnings' ]
	! !

!WADeprecation methodsFor: 'actions' stamp: 'pmm 10/11/2008 18:30'!
resetDeprecationWarnings
	filter reset.
	self answer! !

!WADeprecation methodsFor: 'accessing' stamp: 'lr 12/30/2008 15:37'!
table
	^ table ifNil: [ table := self buildTable ]! !

WATool subclass: #WAInspector
	instanceVariableNames: 'path selected input'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAInspector commentStamp: 'jf 3/3/2009 22:33' prior: 0!
This is an abstract implementation of a web-based object inspector. Platforms should implement their own subclasses, specifying behaviour for all unimplemented methods.

Note that #openNativeInspectorOn: on the class-side also needs to be implemented.

Also subclasses probably want to implement #initialize and #unload on the class-side to call 'self select' and 'self unselect' respectively. This will ensure they are registered as the current implementation when they are loaded.!

!WAInspector class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:35'!
current
	^ Current! !

!WAInspector class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:05'!
current: anInspectorClass
	Current := anInspectorClass! !

!WAInspector class methodsFor: 'instance creation' stamp: 'jf 3/3/2009 22:26'!
on: anObject
	^ self new object: anObject; yourself! !

!WAInspector class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:04'!
select
	self current: self! !

!WAInspector class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:04'!
unselect
	(self current = self) ifTrue: [ self current: nil ]! !

!WAInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 22:10'!
actions
	"A list of action selectors used to render buttons in the Inspector."

	^ #(doIt inspectIt)! !

!WAInspector methodsFor: 'hooks' stamp: 'avi 2/10/2004 16:34'!
children
	^ Array with: path! !

!WAInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 22:10'!
chooseObject: anObject named: aString
	"Choose a new object to be inspected, adding aString to the navigation path."

	path pushSegment: anObject name: aString.
	selected := nil! !

!WAInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 22:09'!
doIt
	"Execute the code in the textarea. We return the result so subclasses can do something
	with it if desired."
	
	^ self evaluate! !

!WAInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 22:11'!
evaluate
	"Evaluate the code in the textarea and return the result."
	
	self subclassResponsibility! !

!WAInspector methodsFor: 'private' stamp: 'jf 2/6/2010 04:32'!
fieldsFor: anObject
	"Return a sequenceable collection of Associations representing labels and values
	of instance variables, etc. which should be displayed when inspecting anObject."

	self subclassResponsibility! !

!WAInspector methodsFor: 'initialization' stamp: 'al 9/7/2006 15:22'!
initialize
	super initialize.
	path := WAPath new! !

!WAInspector methodsFor: 'actions' stamp: 'jf 9/30/2009 01:10'!
inspectIt
	"Evaluate the code in the textarea and then inspect the result."
	| result |
	result := self evaluate.
	self 
		chooseObject: result
		named: result greaseString! !

!WAInspector methodsFor: 'testing' stamp: 'jf 3/3/2009 22:14'!
isPrimitiveType: anObject
	"Return a boolean indicating whether anObject is a primitive type in the system.
	Primitive types will not be rendered with a link allowing them to be inspected."

	^ anObject isNumber or: [ #(nil true false) includes: anObject ]! !

!WAInspector methodsFor: 'accessing' stamp: 'ab 5/31/2003 15:12'!
object
	^ path currentSegment! !

!WAInspector methodsFor: 'accessing' stamp: 'jf 9/30/2009 01:10'!
object: anObject 
	self 
		chooseObject: anObject
		named: anObject greaseString! !

!WAInspector methodsFor: 'rendering' stamp: 'lr 10/28/2007 14:42'!
renderContentOn: html
	html heading
		level: 1;
		with: (self object printStringLimitedTo: 50).
	html render: path.
	html form: [
		self renderMembersOn: html.
		self renderEditAreaOn: html ]! !

!WAInspector methodsFor: 'rendering' stamp: 'jf 3/3/2009 21:59'!
renderEditAreaOn: html
	html textArea
		value: input;
		callback: [ :value | input := value ].
	html break.
	self actions do: [ :each |
		html submitButton
			callback: [ self perform: each ];
			text: each ]! !

!WAInspector methodsFor: 'rendering' stamp: 'jf 2/6/2010 04:32'!
renderMembersOn: html
	| members radioGroup |
	selected := nil.
	members := self fieldsFor: self object.
	members isEmpty ifTrue: [ ^ self ].
	radioGroup := html radioGroup.
	radioGroup
		callback: [ :value | selected := value ];
		selected: selected.
	html table: [
		members do: [ :association |
			self
				renderRow: association value
				named: association key
				group: radioGroup
				on: html ] ]! !

!WAInspector methodsFor: 'rendering' stamp: 'obi 12/31/2009 18:48'!
renderRow: anObject named: aString group: aRadioGroup on: html
	| radioButton |
	html tableRow: [
		html tableHeading: [
			radioButton := aRadioGroup radioButton
				ensureId;
				value: aString;
				yourself.
			html label for: radioButton id; with: [
				(self isPrimitiveType: anObject)
					ifTrue: [ html text: aString ]
					ifFalse: [
						html anchor
							callback: [ self chooseObject: anObject named: aString ];
							with: aString ] ] ].
		html tableData: (anObject printStringLimitedTo: 100) ]! !

!WAInspector methodsFor: 'hooks' stamp: 'lr 5/28/2007 19:20'!
style
	^ 'form {
	padding: 2px;
	margin: 0;
}
form th {
	text-align: left;
	padding-right: 2em;
}
form textarea {
	width: 100%;
	height: 100px;
	border: 1px solid #aaa;
}'! !

WATool subclass: #WAPluggableTool
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAPluggableTool commentStamp: 'pmm 11/14/2008 22:28' prior: 0!
A WAPluggableTool is a wrapper component around tools that can't be made subclasses of WATool. WATool is only really needed to provide the icon image.

Instance Variables
	component:		<WAComponent>

component
	- the component to render
!

!WAPluggableTool class methodsFor: 'instance creation' stamp: 'lr 7/25/2011 19:48'!
on: aComponent
	^ self basicNew
		initializeWith: aComponent;
		yourself! !

!WAPluggableTool methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:07'!
children
	^ Array with: component! !

!WAPluggableTool methodsFor: 'initialization' stamp: 'jf 3/4/2009 21:39'!
initializeWith: aComponent
	self initialize.
	component := aComponent.
	component onAnswer: [ self answer ]! !

!WATool methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	WAToolFiles default updateRoot: anHtmlRoot.
	WADevelopmentFiles default updateRoot: anHtmlRoot! !

WAComponent subclass: #WAWalkback
	instanceVariableNames: 'exception limit'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAWalkback commentStamp: 'pmm 2/18/2011 23:03' prior: 0!
An abstract component that renders an exception stack with temporary variables. Platforms should implement a concrete subclass and register that subclass when it is loaded.

By default this class renders only the top few stack frames, but offers the option to render all. It also offers the option to open a debugger in the image or to proceed with the execution and answers true and false, respectively.

!

!WAWalkback class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:36'!
current
	^ Current! !

!WAWalkback class methodsFor: 'registration' stamp: 'jf 2/20/2009 23:19'!
current: aWalkbackClass
	Current := aWalkbackClass! !

!WAWalkback class methodsFor: 'instance creation' stamp: 'jf 3/3/2009 22:26'!
exception: anException
	^ self basicNew initializeWithException: anException; yourself! !

!WAWalkback class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:26'!
select
	self current: self! !

!WAWalkback class methodsFor: 'registration' stamp: 'jf 3/3/2009 22:26'!
unselect
	(self current = self) ifTrue: [ self current: nil ]! !

!WAWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:42'!
defaultLimit
	"The default number of frames to display in the walkback."
	
	^ 5! !

!WAWalkback methodsFor: 'accessing' stamp: 'lr 6/5/2007 19:33'!
exception
	^ exception! !

!WAWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:54'!
exceptionDescription
	"Return a short string describing the exception."

	| e |
	^ String streamContents: [ :stream |
		e := self exception.
		stream nextPutAll: e class name.
		e messageText isNil ifFalse: [
			stream nextPutAll: ': '; nextPutAll: e messageText ] ]! !

!WAWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 22:54'!
frameCount
	"Return the number of frames or contexts in the walkback."
	
	self subclassResponsibility! !

!WAWalkback methodsFor: 'initialization' stamp: 'jf 2/20/2009 23:40'!
initializeWithException: anException
	self initialize.
	exception := anException! !

!WAWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:43'!
limit
	"The maximum number of frames to display."

	^ limit ifNil: [ limit := self defaultLimit min: self frameCount ]! !

!WAWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:05'!
limit: anInteger
	limit := anInteger! !

!WAWalkback methodsFor: 'rendering' stamp: 'lr 6/5/2007 21:17'!
renderContentOn: html
	self renderHeadingOn: html.
	self renderToolsOn: html.
	self renderExceptionOn: html.
	self renderStackOn: html! !

!WAWalkback methodsFor: 'rendering' stamp: 'jf 2/20/2009 18:09'!
renderExceptionOn: html
	| problems |
	problems := self exception seasidePossibleCauses.
	problems isEmpty ifTrue: [ ^ self ].
	html heading level: 3; with: 'Possible Causes'.
	html unorderedList list: problems! !

!WAWalkback methodsFor: 'rendering' stamp: 'jf 2/20/2009 23:44'!
renderHeadingOn: html
	html heading level: 1; with: 'Seaside Walkback'.
	html heading level: 2; with: self exceptionDescription! !

!WAWalkback methodsFor: 'rendering' stamp: 'jf 2/20/2009 23:13'!
renderStackOn: html
	"Render the walkback itself"
	
	self subclassResponsibility! !

!WAWalkback methodsFor: 'rendering' stamp: 'jf 2/20/2009 23:06'!
renderToolsOn: html
	html paragraph: [
		html anchor
			callback: [ self answer: true ];
			title: 'Open a debugger in your IDE and block until you hit "Proceed"';
			with: 'Debug'.
		self exception isResumable ifTrue: [
			html space.
			html anchor
				callback: [ self answer: false ];
				with: 'Proceed' ].
		self limit < self frameCount ifTrue: [
			html space.
			html anchor
				callback: [ self limit: self frameCount ];
				with: 'Full Stack' ] ]! !

!WAWalkback methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	anHtmlRoot title: self exceptionDescription.
	anHtmlRoot bodyAttributes addClass: 'walkback'.
	WAToolFiles default updateRoot: anHtmlRoot! !

WARequestFilter subclass: #WADeprecatedToolFilter
	instanceVariableNames: 'notifications'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Filter'!

!WADeprecatedToolFilter methodsFor: 'actions' stamp: 'lr 12/30/2008 15:41'!
deprecated: aNotification
	"Adds a deprecated warning aNotification to the receiver. Remove old deprecation warnings that have the same message and detail text."

	notifications := notifications reject: [ :each | 
		(each messageText = aNotification messageText)
			and: [ each details = aNotification details ] ].
	notifications add: aNotification.
	aNotification resume! !

!WADeprecatedToolFilter methodsFor: 'processing' stamp: 'jf 9/30/2009 00:21'!
handleFiltered: aRequestContext 
	[ super handleFiltered: aRequestContext ] 
		on: GRDeprecatedApiNotification
		do: [ :notification | self deprecated: notification ]! !

!WADeprecatedToolFilter methodsFor: 'initialization' stamp: 'lr 9/25/2008 11:25'!
initialize
	super initialize.
	self reset! !

!WADeprecatedToolFilter methodsFor: 'accessing' stamp: 'lr 9/25/2008 15:13'!
notifications
	^ notifications! !

!WADeprecatedToolFilter methodsFor: 'actions' stamp: 'lr 9/25/2008 15:12'!
reset
	notifications := OrderedCollection new! !

WARequestFilter subclass: #WATimingToolFilter
	instanceVariableNames: 'startTime previousTiming'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Filter'!

!WATimingToolFilter methodsFor: 'accessing' stamp: 'lr 9/25/2008 15:29'!
currentTiming
	"Answer the running timing, or 0 if no value is available."

	^ startTime isNil ifTrue: [ 0 ] ifFalse: [ Time millisecondsSince: startTime ]! !

!WATimingToolFilter methodsFor: 'processing' stamp: 'jf 1/14/2009 09:45'!
handleFiltered: aRequestContext
	startTime := Time millisecondClockValue.
	[ super handleFiltered: aRequestContext ]
		ensure: [ previousTiming := self currentTiming ]! !

!WATimingToolFilter methodsFor: 'accessing' stamp: 'lr 9/25/2008 15:29'!
previousTiming
	"Answer the previous timing, or 0 if no value is available."

	^ previousTiming ifNil: [ 0 ]! !

!WAPresenterGuide methodsFor: '*seaside-development' stamp: 'jf 2/12/2009 20:13'!
addHaloVisitor: aHaloVisitor
	client := client addHaloVisitor: aHaloVisitor.
	^ self! !

WAObject subclass: #WAHalo
	instanceVariableNames: 'root mode plugins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAHalo class methodsFor: 'instance creation' stamp: 'jf 2/12/2009 20:22'!
on: aComponent
	^ self basicNew initializeOn: aComponent; yourself! !

!WAHalo methodsFor: 'initialization' stamp: 'jf 2/12/2009 20:22'!
initializeOn: aComponent
	self initialize.
	root := aComponent.
	mode := WAValueHolder with: self modes first.
	plugins := WAHaloPlugin allSubclassesOn: self root for: self! !

!WAHalo methodsFor: 'accessing' stamp: 'lr 12/6/2006 20:25'!
mode
	^ mode contents! !

!WAHalo methodsFor: 'accessing' stamp: 'lr 12/6/2006 20:24'!
mode: aSymbol
	mode contents: aSymbol! !

!WAHalo methodsFor: 'accessing' stamp: 'jf 9/30/2009 01:10'!
modeSelector
	^ (self mode greaseString , ':visitor:') asSymbol! !

!WAHalo methodsFor: 'accessing' stamp: 'jf 2/13/2009 15:36'!
modes
	"Returns a list of possible display modes. These are selectors that are used to render the
	object we are wrapping. The current mode is selected by WAModeHaloPlugin."

	^ #( render source )! !

!WAHalo methodsFor: 'accessing' stamp: 'lr 5/11/2007 01:06'!
plugins
	^ plugins! !

!WAHalo methodsFor: 'rendering-modes' stamp: 'jf 2/13/2009 15:31'!
render: html visitor: aPainterVisitor
	"Render the object we are wrapping normally. See #modes."
	
	aPainterVisitor visit: self root! !

!WAHalo methodsFor: 'rendering' stamp: 'as 2/6/2011 22:05'!
renderWithContext: aRenderContext visitor: aPainterVisitor
	"Render the halo using aRenderContext. aPainterVisitor is the visitor that the halo
	visitor is wrapping; calling #visit: on it will perform the action that would have taken
	place had the halo wrapper not been present.
	"
	| html |
	html := WAHtmlCanvas context: aRenderContext.
	html div class: 'wa-halo'; with: [
		html div class: 'wa-halo-header'; with: [
			(self plugins select: [ :each | each isActive ])
				do: [ :each | html render: each ]
				separatedBy: [ html text: ' ' ] ].
		html div class: 'wa-halo-body'; with: [
			self perform: self modeSelector with: html with: aPainterVisitor ] ].
	html flush! !

!WAHalo methodsFor: 'accessing' stamp: 'lr 5/11/2007 09:39'!
root
	^ root! !

!WAHalo methodsFor: 'rendering-modes' stamp: 'lr 3/7/2009 14:46'!
source: html visitor: aPainterVisitor
	"Render the HTML source of the object we are wrapping using a pretty printed
	document. See #modes.
	Because we are using a new RenderContext, we can't use the passed-in visitor.
	Since we know we want to immediately render, we create a new RenderVisitor to get
	things rolling. Rendering will then continue with the visitors in the context."
	
	| context |
	html div class: 'wa-halo-source'; with: [
		context := html context copy.
		context disableHalos.

		context document: (WAPrettyPrintedDocument context: html context).
		(WARenderVisitor context: context) start: self root.
		context document close ]! !

!WAHalo methodsFor: 'updating' stamp: 'jf 2/13/2009 09:23'!
updateStates: aSnapshot
	"This class is not a Presenter but WAToolDecoration will make sure we get a chance
	to update the snapshot whenever it does."
	
	aSnapshot register: mode! !

WAHtmlDocument subclass: #WAPrettyPrintedDocument
	instanceVariableNames: 'html indentLevel indentCloseTags firstLine singleEncoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Development-Core'!

!WAPrettyPrintedDocument commentStamp: '<historical>' prior: 0!
This Document can be used in place of a WAHtmlDocument and will display a pretty version of the HTML source that would have been generated by the WAHtmlDocument.

The technique here is that:
 - anything we are asked to output raw (i.e. #nextPut: and #nextPutAll:) we encode with entities
 - anything we are asked to entity encode (i.e. via #xmlEncoder) we double-encode, by wrapping the two WAXmlEncoders around each other
 - anything we want to output ourselves (i.e. prettiness) we do using a WAHtmlCanvas that we create ourselves!

!WAPrettyPrintedDocument class methodsFor: 'instance creation' stamp: 'jf 2/9/2011 23:16'!
context: aRenderContext
	^ self basicNew initializeWithContext: aRenderContext; yourself! !

!WAPrettyPrintedDocument class methodsFor: 'instance creation' stamp: 'jf 2/9/2011 23:03'!
on: aStream codec: aCodec
	self error: 'Use #context:'! !

!WAPrettyPrintedDocument class methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
renderer: aRenderer 
	self 
		greaseDeprecatedApi: 'WAPrettyPrintedDocument class>>#renderer:'
		details: 'Use WAPrettyPrintedDocument class>>#context:.'.
	^ self context: aRenderer context! !

!WAPrettyPrintedDocument methodsFor: 'private' stamp: 'jf 2/9/2011 23:41'!
attribute: aKey with: aValue 
	aValue == false ifTrue: [ ^ self ].
	html space.
	html span
		class: 'wa-attribute-key';
		with: aKey.
	self nextPut: $=.
	html span
		class: 'wa-attribute-value';
		with: [
			self nextPut: $".
			(self knownRefs includes: aKey) 
				ifTrue: [
					html anchor
						url: aValue greaseString;
						with: [ self print: aValue ] ]
				ifFalse: [
					(self knownEvents includes: aKey) 
						ifTrue: [
							html anchor
								onClick: aValue;
								with: [ self print: aValue ] ]
						ifFalse: [
							aValue == true 
								ifTrue: [ self nextPutAll: aKey ]
								ifFalse: [ self print: aValue ] ] ].
			self nextPut: $" ]! !

!WAPrettyPrintedDocument methodsFor: 'private' stamp: 'pmm 2/9/2011 07:03'!
attributes: anAttributes
	anAttributes ifNil: [ ^ self ].
	anAttributes keysAndValuesDo: [ :key :value |
		self attribute: key with: value ]! !

!WAPrettyPrintedDocument methodsFor: 'writing' stamp: 'jf 2/9/2011 23:33'!
closeTag: aString
	indentLevel := indentLevel - 1.
	indentCloseTags removeLast
		ifTrue: [ self indent ].
	self nextPutAll: '</'.
	html span 
		class: 'wa-tag'; 
		with: aString.
	self nextPut: $>! !

!WAPrettyPrintedDocument methodsFor: 'private' stamp: 'jf 2/9/2011 23:50'!
indent
	firstLine
		ifTrue: [ firstLine := false ]
		ifFalse: [ html break ].
	indentLevel timesRepeat: [ html space: 2 ].
	html flush! !

!WAPrettyPrintedDocument methodsFor: 'initialization' stamp: 'jf 2/9/2011 23:16'!
initializeWithContext: aRenderContext
	self initializeWithStream: aRenderContext document stream codec: self requestContext codec.
	xmlEncoder := WAXmlEncoder on: (singleEncoder := xmlEncoder).
	html := WAHtmlCanvas context: aRenderContext.
	indentCloseTags := OrderedCollection with: false.
	indentLevel := 0.
	firstLine := true! !

!WAPrettyPrintedDocument methodsFor: 'configuration' stamp: 'pmm 6/25/2011 20:47'!
knownEvents
	^ #('onabort' 'onblur' 'onchange' 'onclick' 'ondblclick' 'onerror' 'onfocus' 'oninput' 'onkeydown' 'onkeypress' 'onkeyup' 'onload' 'onmousedown' 'onmousemove' 'onmouseout' 'onmouseover' 'onmouseup' 'onreset' 'onresize' 'onscroll' 'onselect' 'onsubmit' 'onunload')! !

!WAPrettyPrintedDocument methodsFor: 'configuration' stamp: 'lr 4/20/2007 22:11'!
knownRefs
	^ #('action' 'cite' 'classid' 'codebase' 'data' 'href' 'longdesc' 'profile' 'src' 'usemap')! !

!WAPrettyPrintedDocument methodsFor: 'writing' stamp: 'jf 2/9/2011 23:05'!
nextPut: aCharacter
	singleEncoder nextPut: aCharacter! !

!WAPrettyPrintedDocument methodsFor: 'writing' stamp: 'jf 2/9/2011 23:05'!
nextPutAll: aString
	singleEncoder nextPutAll: aString! !

!WAPrettyPrintedDocument methodsFor: 'writing' stamp: 'jf 2/9/2011 23:34'!
openTag: aString attributes: anAttributes closed: aBoolean
	self indent.
	self nextPut: $<.
	html span
		class: 'wa-tag';
		with: aString.
	self attributes: anAttributes.
	aBoolean ifTrue: [ self nextPut: $/ ].
	self nextPut: $>.
	aBoolean ifFalse: [ indentLevel := indentLevel + 1 ].
	indentCloseTags removeLast; addLast: true; addLast: aBoolean! !

!WAAdmin class methodsFor: '*seaside-development' stamp: 'jf 5/23/2010 15:36'!
developmentToolsEnabled
	^ self applicationDefaults parents includes: WADevelopmentConfiguration instance! !

!WAAdmin class methodsFor: '*seaside-development' stamp: 'jf 5/23/2010 15:26'!
disableDevelopmentTools
	self applicationDefaults removeParent: WADevelopmentConfiguration instance! !

!WAAdmin class methodsFor: '*seaside-development' stamp: 'jf 5/23/2010 15:26'!
enableDevelopmentTools
	self applicationDefaults addParent: WADevelopmentConfiguration instance! !
WAWalkbackErrorHandler initialize!
WADevelopmentConfiguration initialize!
